﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using Infragistics.Win.UltraWinGrid;
using System.Collections;

namespace AnimatedGifSample
{
    public partial class Form1 : Form
    {
        private DataService _dataService;

        #region ctor and load
        public Form1()
        {
            InitializeComponent();
            _dataService = new DataService();
            _dataService.GetChildRowsCompleted += new EventHandler<DataService.ChildRowsEventArgs>(_dataService_GetChildRowsCompleted);
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            BindingList<BlotterRow> data = new BindingList<BlotterRow>();
            data.Add(new BlotterRow("First Row"));

            _grid.DataSource = data;
        }
        #endregion

        #region helpers
        private UltraGridRow findRow(BlotterRow bRow)
        {
            foreach (object objRow in _grid.Rows.GetRowEnumerator(GridRowType.DataRow, null, null))
            {
                UltraGridRow row = objRow as UltraGridRow;
                BlotterRow foundRow = row.ListObject as BlotterRow;
                if (foundRow.Id == bRow.Id)
                {
                    return row;
                }
            }
            return null;
        }
        #endregion

        private void _grid_BeforeRowExpanded(object sender, Infragistics.Win.UltraWinGrid.CancelableRowEventArgs e)
        {
            if (e.Row.IsDataRow && !(bool)e.Row.Cells["IsLoaded"].Value) 
            {
                SetRowLoading(e.Row, true);
                _dataService.GetChildRowsAsync(e.Row.ListObject as BlotterRow);
            }
        }

        private void _dataService_GetChildRowsCompleted(object sender, DataService.ChildRowsEventArgs e)
        {
            if (e.ParentRow != null)
            {
                UltraGridRow gridRow = findRow(e.ParentRow);
                if (gridRow != null)
                {
                    e.ParentRow.IsLoaded = true;
                    foreach (BlotterRow child in e.ChildRows)
                    {
                        e.ParentRow.ChildRows.Add(child);
                    }
                    SetRowLoading(gridRow, false);
                }
            }
        }

        public void SetRowLoading(UltraGridRow row, bool loading)
        {
            if (loading)
            {
                row.RowSelectorAppearance.Image = Properties.Resources.loading;
                EventHandler handler = new EventHandler(image_onFrameChange);
                row.Tag = handler;
                ImageAnimator.Animate((Image)row.RowSelectorAppearance.Image, handler);
            }
            else
            {
                if (row != null && row.RowSelectorAppearance.Image != null)
                {
                    ImageAnimator.StopAnimate((Image)row.RowSelectorAppearance.Image, row.Tag as EventHandler); //new EventHandler(image_onFrameChange));
                    row.Tag = null;
                    row.RowSelectorAppearance.ResetImage();
                }
            }
        }

        private void image_onFrameChange(object sender, EventArgs e)
        {
            if (this.InvokeRequired)
            {
                _grid.Invoke(new EventHandler(image_onFrameChange), new object[] { sender, e });
            }
            else
            {
                _grid.Invalidate(true);
                ImageAnimator.UpdateFrames((Image)sender);
            }
        }
    }

    #region BlotterRow class
    public class BlotterRow
    {
        private static Random _random = new Random();

        public int Id { get; set; }
        public string Name { get; set; }
        public BindingList<BlotterRow> ChildRows { get; set; }
        public bool IsLoaded { get; set; }

        public BlotterRow(string name)
        {
            Id = _random.Next();
            Name = name;
            ChildRows = new BindingList<BlotterRow>();
        }
    }
    #endregion

    #region DataService
    public class DataService
    {
        private delegate BindingList<BlotterRow> GetChildRowsHandler();
        public event EventHandler<ChildRowsEventArgs> GetChildRowsCompleted;
        
        public void GetChildRowsAsync(BlotterRow parent)
        {
            GetChildRowsHandler handler = new GetChildRowsHandler(GetChildRows);
            handler.BeginInvoke(new AsyncCallback(GetChildRowsCallback), new AsyncState(handler, parent));
        }

        public BindingList<BlotterRow> GetChildRows()
        {
            System.Threading.Thread.Sleep(1500);
            Random rand = new Random();
            int numRows = rand.Next(1, 5);
            BindingList<BlotterRow> children = new BindingList<BlotterRow>();
            for (int i = 0; i < numRows; i++)
            {
                children.Add(new BlotterRow("Child Row " + i));
            }
            return children;
        }

        private void GetChildRowsCallback(IAsyncResult result)
        {
            AsyncState state = (AsyncState)result.AsyncState;
            GetChildRowsHandler handler = state.Handler as GetChildRowsHandler;
            BindingList<BlotterRow> rows = handler.EndInvoke(result);

            if (GetChildRowsCompleted != null)
            {
                GetChildRowsCompleted(this, new ChildRowsEventArgs(state.State as BlotterRow, rows));
            }
        }

        public class ChildRowsEventArgs : EventArgs
        {
            public BlotterRow ParentRow { get; set; }
            public BindingList<BlotterRow> ChildRows { get; set; }
            public ChildRowsEventArgs(BlotterRow parent, BindingList<BlotterRow> children)
            {
                ParentRow = parent;
                ChildRows = children;
            }
        }

        public class AsyncState
        {
            public Delegate Handler { get; set; }
            public object State { get; set; }

            public AsyncState(Delegate handler, object state)
            {
                Handler = handler;
                State = state;
            }
        }
    }
    #endregion
}
